//---------------------------------------------------------------------------
// postmain.cpp : rtkpost main (rtklib post-processing analysis)
//
//			Copyright (C) 2007 by T.TAKASU, All rights reserved.
//
// version : $Revision: 1.1 $ $Date: 2008/07/17 22:14:45 $
// history : 2008/07/14  1.0 new
//           2008/11/17  1.1 v.2.1.1
//---------------------------------------------------------------------------
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <vcl.h>
#pragma hdrstop

#include "postmain.h"
#include "postopt.h"
#include "kmzconv.h"
#include "refdlg.h"
#include "viewer.h"
#include "confdlg.h"
#include "timedlg.h"
#include "aboutdlg.h"
#include "rtklib.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMainForm *MainForm;

#define PRGNAME		"RTKPOST"
#define MAXHIST		20
#define GOOGLE_EARTH "C:\\Program Files\\Google\\Google Earth\\googleearth.exe"

static const char version[]="$Revision: 1.1 $ $Date: 2008/07/17 22:14:45 $";

//---------------------------------------------------------------------------
extern "C" {
static gtime_t tstart_={0};
static gtime_t tend_  ={0};
static char rov_ [256]="";
static char base_[256]="";

static void setrov (char *rov ) {strcpy(rov_ ,rov );}
static void setbase(char *base) {strcpy(base_,base);}
extern int showmsg(char *format, ...)
{
	va_list arg;
	char buff[1024],*p=buff;
	va_start(arg,format);
	p+=vsprintf(p,format,arg);
	va_end(arg);
	if (*rov_&&*base_) sprintf(p," (%s-%s)",rov_,base_);
	else if (*rov_ )   sprintf(p," (%s)",rov_ );
	else if (*base_)   sprintf(p," (%s)",base_);
	MainForm->ShowMsg(buff);
	if (strncmp(buff,"processing",10)) Application->ProcessMessages();
	return !MainForm->BtnExec->Enabled;
}
extern void settspan(gtime_t ts, gtime_t te)
{
	tstart_=ts;
	tend_  =te;
}
extern void settime(gtime_t time)
{
	static int i=0;
	double tt=timediff(tend_,tstart_);
	if (tt>0.0) {
		MainForm->Progress->Position=(int)(timediff(time,tstart_)/tt*100.0+0.5);
	}
	if (i++%23==0) Application->ProcessMessages();
}
}
//---------------------------------------------------------------------------
static double str2dbl(AnsiString str)
{
	double val=0.0;
	sscanf(str.c_str(),"%lf",&val);
	return val;
}
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
	: TForm(Owner)
{
	DynamicModel=IonoEst=TropEst=RovAntPcv=RefAntPcv=AmbRes=0;
	RovPosType=RefPosType=0;
	OutCntResetBias=5; LockCntFixAmb=5;
	MaxAgeDiff=30.0; RejectThres=30.0;
	MeasErr1=100.0; MeasErr2=0.004; MeasErr3=0.003; MeasErr4=1.0;
	SatClkStab=1E-11; ValidThresAR=3.0;
	RovAntE=RovAntN=RovAntU=RefAntE=RefAntN=RefAntU=0.0;
	for (int i=0;i<3;i++) RovPos[i]=0.0;
	for (int i=0;i<3;i++) RefPos[i]=0.0;
	DoubleBuffered=true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCreate(TObject *Sender)
{
	::DragAcceptFiles(Handle,true);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormShow(TObject *Sender)
{
	TIniFile *ini=new TIniFile("rtkpost.ini");
	TimeStart->Checked =ini->ReadInteger("set","timestart",   0);
	TimeEnd->Checked   =ini->ReadInteger("set","timeend",     0);
	TimeY1->Text	   =ini->ReadString ("set","timey1",      "2000/01/01");
	TimeY1->Text	   =ini->ReadString ("set","timey1",      "2000/01/01");
	TimeH1->Text	   =ini->ReadString ("set","timeh1",      "00:00:00");
	TimeY2->Text	   =ini->ReadString ("set","timey2",      "2000/01/01");
	TimeH2->Text	   =ini->ReadString ("set","timeh2",      "00:00:00");
	TimeIntF ->Checked =ini->ReadInteger("set","timeintf",    0);
	TimeInt->Text	   =ini->ReadString ("set","timeint",     "0");
	TimeUnitF->Checked =ini->ReadInteger("set","timeunitf",   0);
	TimeUnit->Text	   =ini->ReadString ("set","timeunit",    "24");
	InputFile1->Text   =ini->ReadString ("set","inputfile1",  "");
	InputFile2->Text   =ini->ReadString ("set","inputfile2",  "");
	InputFile3->Text   =ini->ReadString ("set","inputfile3",  "");
	InputFile4->Text   =ini->ReadString ("set","inputfile4",  "");
	OutputFile->Text   =ini->ReadString ("set","outputfile",  "");
	
	InputFile1->Items  =ReadList(ini,"hist","inputfile1");
	InputFile2->Items  =ReadList(ini,"hist","inputfile2");
	InputFile3->Items  =ReadList(ini,"hist","inputfile3");
	InputFile4->Items  =ReadList(ini,"hist","inputfile4");
	OutputFile->Items  =ReadList(ini,"hist","outputfile");
	
	PosMode			   =ini->ReadInteger("opt","posmode",     2);
	Freq			   =ini->ReadInteger("opt","freq",        1);
	Solution		   =ini->ReadInteger("opt","solution",    0);
	ElMask			   =ini->ReadFloat	("opt","elmask",      15.0);
	SnrMask			   =ini->ReadFloat	("opt","snrmask",     0.0);
	IonoEst			   =ini->ReadInteger("opt","ionoest",     0);
	TropEst			   =ini->ReadInteger("opt","tropest",     0);
	RcvBiasEst		   =ini->ReadInteger("opt","rcvbiasest",  0);
	DynamicModel	   =ini->ReadInteger("opt","dynamicmodel",0);
	SatEphem		   =ini->ReadInteger("opt","satephem",    0);
	ExSats			   =ini->ReadString ("opt","exsats",      "");
	NavSys			   =ini->ReadInteger("opt","navsys",      SYS_GPS);
	
	AmbRes			   =ini->ReadInteger("opt","ambres",      1);
	GloAmbRes		   =ini->ReadInteger("opt","gloambres",   1);
	ValidThresAR	   =ini->ReadFloat	("opt","validthresar",3.0);
	LockCntFixAmb	   =ini->ReadInteger("opt","lockcntfixamb",0);
	ElMaskAR		   =ini->ReadFloat	("opt","elmaskar",    0.0);
	OutCntResetBias    =ini->ReadInteger("opt","outcntresetbias",5);
	SlipThres		   =ini->ReadFloat	("opt","slipthres",   0.05);
	MaxAgeDiff		   =ini->ReadFloat	("opt","maxagediff",  30.0);
	RejectThres		   =ini->ReadFloat	("opt","rejectthres", 30.0);
	NumIter			   =ini->ReadInteger("opt","numiter",     1);
	CodeSmooth		   =ini->ReadInteger("opt","codesmooth",  0);
	BaseLine[0]		   =ini->ReadFloat  ("opt","baselinelen", 0.0);
	BaseLine[1]		   =ini->ReadFloat  ("opt","baselinesig", 0.0);
	BaseLineConst	   =ini->ReadInteger("opt","baselineconst",0);
	
	SolFormat		   =ini->ReadInteger("opt","solformat",   0);
	TimeFormat		   =ini->ReadInteger("opt","timeformat",  1);
	TimeDecimal		   =ini->ReadInteger("opt","timedecimal", 3);
	LatLonFormat	   =ini->ReadInteger("opt","latlonformat",0);
	FieldSep		   =ini->ReadString ("opt","fieldsep",    "");
	OutputHead		   =ini->ReadInteger("opt","outputhead",  1);
	OutputOpt		   =ini->ReadInteger("opt","outputopt",   1);
	OutputDatum		   =ini->ReadInteger("opt","outputdatum", 0);
	OutputHeight	   =ini->ReadInteger("opt","outputheight",0);
	OutputGeoid		   =ini->ReadInteger("opt","outputgeoid", 0);
	DebugTrace		   =ini->ReadInteger("opt","debugtrace",  0);
	DebugStatus		   =ini->ReadInteger("opt","debugstatus", 0);
	
	MeasErr1		   =ini->ReadFloat	("opt","measerr1",    100.0);
	MeasErr2		   =ini->ReadFloat	("opt","measerr2",    0.003);
	MeasErr3		   =ini->ReadFloat	("opt","measerr3",    0.003);
	MeasErr4		   =ini->ReadFloat	("opt","measerr4",    0.000);
	MeasErr5		   =ini->ReadFloat	("opt","measerr5",    10.000);
	SatClkStab		   =ini->ReadFloat	("opt","satclkstab",  5E-12);
	PrNoise1		   =ini->ReadFloat	("opt","prnoise1",    1E-4);
	PrNoise2		   =ini->ReadFloat	("opt","prnoise2",    1E-3);
	PrNoise3		   =ini->ReadFloat	("opt","prnoise3",    1E-4);
	PrNoise4		   =ini->ReadFloat	("opt","prnoise4",    1E+1);
	PrNoise5		   =ini->ReadFloat	("opt","prnoise5",    1E+1);
	
	RovPosType		   =ini->ReadInteger("opt","rovpostype",  0);
	RefPosType		   =ini->ReadInteger("opt","refpostype",  0);
	RovPos[0]		   =ini->ReadFloat	("opt","rovpos1",     0.0);
	RovPos[1]		   =ini->ReadFloat	("opt","rovpos2",     0.0);
	RovPos[2]		   =ini->ReadFloat	("opt","rovpos3",     0.0);
	RefPos[0]		   =ini->ReadFloat	("opt","refpos1",     0.0);
	RefPos[1]		   =ini->ReadFloat	("opt","refpos2",     0.0);
	RefPos[2]		   =ini->ReadFloat	("opt","refpos3",     0.0);
	RovAntPcv		   =ini->ReadInteger("opt","rovantpcv",   0);
	RefAntPcv		   =ini->ReadInteger("opt","refantpcv",   0);
	RovAnt			   =ini->ReadString ("opt","rovant",      "");
	RefAnt			   =ini->ReadString ("opt","refant",      "");
	RovAntE			   =ini->ReadFloat	("opt","rovante",     0.0);
	RovAntN			   =ini->ReadFloat	("opt","rovantn",     0.0);
	RovAntU			   =ini->ReadFloat	("opt","rovantu",     0.0);
	RefAntE			   =ini->ReadFloat	("opt","refante",     0.0);
	RefAntN			   =ini->ReadFloat	("opt","refantn",     0.0);
	RefAntU			   =ini->ReadFloat	("opt","refantu",     0.0);
	
	AntPcvFile		   =ini->ReadString ("opt","antpcvfile",  "");
	IntpRefObs		   =ini->ReadInteger("opt","intprefobs",  0);
	NetRSCorr		   =ini->ReadInteger("opt","netrscorr",   0);
	SatClkCorr		   =ini->ReadInteger("opt","satclkcorr",  0);
	NetRSCorrFile1	   =ini->ReadString ("opt","netrscorrfile1","");
	NetRSCorrFile2	   =ini->ReadString ("opt","netrscorrfile2","");
	SatClkCorrFile	   =ini->ReadString ("opt","satclkcorrfile","");
	SbasCorr		   =ini->ReadInteger("opt","sbascorr",    0);
	SbasCorr1		   =ini->ReadInteger("opt","sbascorr1",   0);
	SbasCorr2		   =ini->ReadInteger("opt","sbascorr2",   0);
	SbasCorr3		   =ini->ReadInteger("opt","sbascorr3",   0);
	SbasCorr4		   =ini->ReadInteger("opt","sbascorr4",   0);
	SbasCorrFile	   =ini->ReadString ("opt","sbascorrfile","");
	PrecEphFile		   =ini->ReadString ("opt","precephfile", "");
	SatPcvFile		   =ini->ReadString ("opt","satpcvfile",  "");
	StaPosFile		   =ini->ReadString ("opt","staposfile",  "");
	GeoidDataFile      =ini->ReadString ("opt","geoiddatafile","");
	GoogleEarthFile    =ini->ReadString ("opt","googleearthfile",GOOGLE_EARTH);
	
	RovList            =ini->ReadString ("opt","rovlist",     "");
	for (char *p=RovList.c_str();*p;p++) {
		if ((p=strstr(p,"@@"))) strncpy(p,"\r\n",2); else break;
	}
	BaseList           =ini->ReadString ("opt","baselist",    "");
	for (char *p=BaseList.c_str();*p;p++) {
		if ((p=strstr(p,"@@"))) strncpy(p,"\r\n",2); else break;
	}
	ConvDialog->TimeSpan->Checked	 =ini->ReadInteger("conv","timespan",  0);
	ConvDialog->TimeIntF->Checked	 =ini->ReadInteger("conv","timeintf",  0);
	ConvDialog->TimeY1->Text		 =ini->ReadString ("conv","timey1",    "2000/01/01");
	ConvDialog->TimeH1->Text		 =ini->ReadString ("conv","timeh1",    "00:00:00");
	ConvDialog->TimeY2->Text		 =ini->ReadString ("conv","timey2",    "2000/01/01");
	ConvDialog->TimeH2->Text		 =ini->ReadString ("conv","timeh2",    "00:00:00");
	ConvDialog->TimeInt->Text		 =ini->ReadString ("conv","timeint",   "0");
	ConvDialog->TrackColor->ItemIndex=ini->ReadInteger("conv","trackcolor",5);
	ConvDialog->PointColor->ItemIndex=ini->ReadInteger("conv","pointcolor",5);
	ConvDialog->OutputAlt->ItemIndex =ini->ReadInteger("conv","outputalt", 0);
	ConvDialog->OutputTime->ItemIndex=ini->ReadInteger("conv","outputtime",0);
	ConvDialog->AddOffset->Checked	 =ini->ReadInteger("conv","addoffset", 0);
	ConvDialog->Offset1->Text		 =ini->ReadString ("conv","offset1",   "0");
	ConvDialog->Offset2->Text		 =ini->ReadString ("conv","offset2",   "0");
	ConvDialog->Offset3->Text		 =ini->ReadString ("conv","offset3",   "0");
	ConvDialog->Compress->Checked	 =ini->ReadInteger("conv","compress",  0);

	TTextViewer::Color1=(TColor)ini->ReadInteger("viewer","color1",(int)clBlack);
	TTextViewer::Color2=(TColor)ini->ReadInteger("viewer","color2",(int)clWhite);
	TTextViewer::FontD=new TFont;
	TTextViewer::FontD->Name=ini->ReadString("viewer","fontname","Courier New");
	TTextViewer::FontD->Size=ini->ReadInteger("viewer","fontsize",9);
	delete ini;
	UpdateEnable();
	AnsiString s;
	Caption=s.sprintf("%s ver.%s",PRGNAME,VER_RTKLIB);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormClose(TObject *Sender, TCloseAction &Action)
{
	TIniFile *ini=new TIniFile("rtkpost.ini");
	ini->WriteInteger("set","timestart",   TimeStart ->Checked?1:0);
	ini->WriteInteger("set","timeend",     TimeEnd   ->Checked?1:0);
	ini->WriteString ("set","timey1",      TimeY1    ->Text);
	ini->WriteString ("set","timeh1",      TimeH1    ->Text);
	ini->WriteString ("set","timey2",      TimeY2    ->Text);
	ini->WriteString ("set","timeh2",      TimeH2    ->Text);
	ini->WriteInteger("set","timeintf",    TimeIntF  ->Checked?1:0);
	ini->WriteString ("set","timeint",     TimeInt   ->Text);
	ini->WriteInteger("set","timeunitf",   TimeUnitF ->Checked?1:0);
	ini->WriteString ("set","timeunit",    TimeUnit  ->Text);
	ini->WriteString ("set","inputfile1",  InputFile1->Text);
	ini->WriteString ("set","inputfile2",  InputFile2->Text);
	ini->WriteString ("set","inputfile3",  InputFile3->Text);
	ini->WriteString ("set","inputfile4",  InputFile4->Text);
	ini->WriteString ("set","outputfile",  OutputFile->Text);

	WriteList(ini,"hist","inputfile1",     InputFile1->Items);
	WriteList(ini,"hist","inputfile2",     InputFile2->Items);
	WriteList(ini,"hist","inputfile3",     InputFile3->Items);
	WriteList(ini,"hist","inputfile4",     InputFile4->Items);
	WriteList(ini,"hist","outputfile",     OutputFile->Items);

	ini->WriteInteger("opt","posmode",     PosMode);
	ini->WriteInteger("opt","freq",        Freq);
	ini->WriteInteger("opt","solution",    Solution);
	ini->WriteFloat  ("opt","elmask",      ElMask);
	ini->WriteFloat  ("opt","snrmask",     SnrMask);
	ini->WriteInteger("opt","ionoest",     IonoEst);
	ini->WriteInteger("opt","tropest",     TropEst);
	ini->WriteInteger("opt","rcvbiasest",  RcvBiasEst);
	ini->WriteInteger("opt","dynamicmodel",DynamicModel);
	ini->WriteInteger("opt","satephem",    SatEphem);
	ini->WriteString ("opt","exsats",      ExSats);
	ini->WriteInteger("opt","navsys",      NavSys);

	ini->WriteInteger("opt","ambres",      AmbRes);
	ini->WriteInteger("opt","gloambres",   GloAmbRes);
	ini->WriteFloat  ("opt","validthresar",ValidThresAR);
	ini->WriteInteger("opt","lockcntfixamb",LockCntFixAmb);
	ini->WriteFloat  ("opt","elmaskar",    ElMaskAR);
	ini->WriteInteger("opt","outcntresetbias",OutCntResetBias);
	ini->WriteFloat  ("opt","slipthres",   SlipThres);
	ini->WriteFloat  ("opt","maxagediff",  MaxAgeDiff);
	ini->WriteFloat  ("opt","rejectthres", RejectThres);
	ini->WriteInteger("opt","numiter",     NumIter);
	ini->WriteInteger("opt","codesmooth",  CodeSmooth);
	ini->WriteFloat  ("opt","baselinelen", BaseLine[0]);
	ini->WriteFloat  ("opt","baselinesig", BaseLine[1]);
	ini->WriteInteger("opt","baselineconst",BaseLineConst);
	
	ini->WriteInteger("opt","solformat",   SolFormat);
	ini->WriteInteger("opt","timeformat",  TimeFormat);
	ini->WriteInteger("opt","timedecimal", TimeDecimal);
	ini->WriteInteger("opt","latlonformat",LatLonFormat);
	ini->WriteString ("opt","fieldsep",    FieldSep);
	ini->WriteInteger("opt","outputhead",  OutputHead);
	ini->WriteInteger("opt","outputopt",   OutputOpt);
	ini->WriteInteger("opt","outputdatum", OutputDatum);
	ini->WriteInteger("opt","outputheight",OutputHeight);
	ini->WriteInteger("opt","outputgeoid", OutputGeoid);
	ini->WriteInteger("opt","debugtrace",  DebugTrace);
	ini->WriteInteger("opt","debugstatus", DebugStatus);

	ini->WriteFloat  ("opt","measerr1",    MeasErr1);
	ini->WriteFloat  ("opt","measerr2",    MeasErr2);
	ini->WriteFloat  ("opt","measerr3",    MeasErr3);
	ini->WriteFloat  ("opt","measerr4",    MeasErr4);
	ini->WriteFloat  ("opt","measerr5",    MeasErr5);
	ini->WriteFloat  ("opt","satclkstab",  SatClkStab);
	ini->WriteFloat  ("opt","prnoise1",    PrNoise1);
	ini->WriteFloat  ("opt","prnoise2",    PrNoise2);
	ini->WriteFloat  ("opt","prnoise3",    PrNoise3);
	ini->WriteFloat  ("opt","prnoise4",    PrNoise4);
	ini->WriteFloat  ("opt","prnoise5",    PrNoise5);
	
	ini->WriteInteger("opt","rovpostype",  RovPosType);
	ini->WriteInteger("opt","refpostype",  RefPosType);
	ini->WriteFloat  ("opt","rovpos1",     RovPos[0]);
	ini->WriteFloat  ("opt","rovpos2",     RovPos[1]);
	ini->WriteFloat  ("opt","rovpos3",     RovPos[2]);
	ini->WriteFloat  ("opt","refpos1",     RefPos[0]);
	ini->WriteFloat  ("opt","refpos2",     RefPos[1]);
	ini->WriteFloat  ("opt","refpos3",     RefPos[2]);
	ini->WriteInteger("opt","rovantpcv",   RovAntPcv);
	ini->WriteInteger("opt","refantpcv",   RefAntPcv);
	ini->WriteString ("opt","rovant",      RovAnt);
	ini->WriteString ("opt","refant",      RefAnt);
	ini->WriteFloat  ("opt","rovante",     RovAntE);
	ini->WriteFloat  ("opt","rovantn",     RovAntN);
	ini->WriteFloat  ("opt","rovantu",     RovAntU);
	ini->WriteFloat  ("opt","refante",     RefAntE);
	ini->WriteFloat  ("opt","refantn",     RefAntN);
	ini->WriteFloat  ("opt","refantu",     RefAntU);
	
	ini->WriteString ("opt","antpcvfile",  AntPcvFile);
	ini->WriteInteger("opt","intprefobs",  IntpRefObs);
	ini->WriteInteger("opt","netrscorr",   NetRSCorr);
	ini->WriteInteger("opt","satclkcorr",  SatClkCorr);
	ini->WriteString ("opt","netrscorrfile1",NetRSCorrFile1);
	ini->WriteString ("opt","netrscorrfile2",NetRSCorrFile2);
	ini->WriteString ("opt","satclkcorrfile",SatClkCorrFile);
	ini->WriteInteger("opt","sbascorr",    SbasCorr);
	ini->WriteInteger("opt","sbascorr1",   SbasCorr1);
	ini->WriteInteger("opt","sbascorr2",   SbasCorr2);
	ini->WriteInteger("opt","sbascorr3",   SbasCorr3);
	ini->WriteInteger("opt","sbascorr4",   SbasCorr4);
	ini->WriteString ("opt","sbascorrfile",SbasCorrFile);
	ini->WriteString ("opt","precephfile", PrecEphFile);
	ini->WriteString ("opt","satpcvfile",  SatPcvFile);
	ini->WriteString ("opt","staposfile",  StaPosFile);
	ini->WriteString ("opt","geoiddatafile",GeoidDataFile);
	ini->WriteString ("opt","googleearthfile",GoogleEarthFile);
	
	for (char *p=RovList.c_str();*p;p++) {
		if ((p=strstr(p,"\r\n"))) strncpy(p,"@@",2); else break;
	}
	ini->WriteString ("opt","rovlist",     RovList);
	
	for (char *p=BaseList.c_str();*p;p++) {
		if ((p=strstr(p,"\r\n"))) strncpy(p,"@@",2); else break;
	}
	ini->WriteString ("opt","baselist",    BaseList);
	
	ini->WriteInteger("conv","timespan",   ConvDialog->TimeSpan->Checked);
	ini->WriteString ("conv","timey1",     ConvDialog->TimeY1->Text);
	ini->WriteString ("conv","timeh1",     ConvDialog->TimeH1->Text);
	ini->WriteString ("conv","timey2",     ConvDialog->TimeY2->Text);
	ini->WriteString ("conv","timeh2",     ConvDialog->TimeH2->Text);
	ini->WriteInteger("conv","timeintf",   ConvDialog->TimeIntF->Checked);
	ini->WriteString ("conv","timeint",    ConvDialog->TimeInt->Text);
	ini->WriteInteger("conv","trackcolor", ConvDialog->TrackColor->ItemIndex);
	ini->WriteInteger("conv","pointcolor", ConvDialog->PointColor->ItemIndex);
	ini->WriteInteger("conv","outputalt",  ConvDialog->OutputAlt->ItemIndex);
	ini->WriteInteger("conv","outputtime", ConvDialog->OutputTime->ItemIndex);
	ini->WriteInteger("conv","addoffset",  ConvDialog->AddOffset->Checked);
	ini->WriteString ("conv","offset1",    ConvDialog->Offset1->Text);
	ini->WriteString ("conv","offset2",    ConvDialog->Offset2->Text);
	ini->WriteString ("conv","offset3",    ConvDialog->Offset3->Text);
	ini->WriteInteger("conv","compress",   ConvDialog->Compress->Checked);
	
	ini->WriteInteger("viewer","color1",(int)TTextViewer::Color1);
	ini->WriteInteger("viewer","color2",(int)TTextViewer::Color2);
	ini->WriteString ("viewer","fontname",TTextViewer::FontD->Name);
	ini->WriteInteger("viewer","fontsize",TTextViewer::FontD->Size);
	delete ini;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DropFiles(TWMDropFiles msg)
{
	POINT point={0};
	char *p,file[1024];
	int top;
	if (::DragQueryFile((HDROP)msg.Drop,0xFFFFFFFF,NULL,0)<=0) return;
	::DragQueryFile((HDROP)msg.Drop,0,file,sizeof(file));
	if (!::DragQueryPoint((HDROP)msg.Drop,&point)) return;
	top=Panel1->Top+Panel4->Top;
	if (point.y<=top+InputFile1->Top+InputFile1->Height) {
		InputFile1->Text=file;
		if (!(p=strrchr(file,'.'))) p=file+strlen(file);
		strcpy(p,".pos");
		OutputFile->Text=file;
	}
	else if (point.y<=top+InputFile2->Top+InputFile2->Height) {
		InputFile2->Text=file;
	}
	else if (point.y<=top+InputFile4->Top+InputFile4->Height) {
		InputFile4->Text=file;
	}
	else if (point.y<=top+InputFile3->Top+InputFile3->Height) {
		InputFile3->Text=file;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::InputFile1Change(TObject *Sender)
{
	char *p,file[1024];
	if (InputFile1->Text=="") return;
	strcpy(file,InputFile1->Text.c_str());
	if (!(p=strrchr(file,'.'))) p=file+strlen(file);
	strcpy(p,".pos");
	OutputFile->Text=file;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnInputPlot1Click(TObject *Sender)
{
	AnsiString files[3];
	char navfile[1024],gnavfile[1024];
	
	files[0]=FilePath(InputFile1->Text);
	files[1]=FilePath(InputFile2->Text);
	files[2]=FilePath(InputFile4->Text);
	if (files[1]=="") {
		if (ObsToNav(files[0].c_str(),navfile)) files[1]=navfile;
	}
	if (files[2]=="") {
		if (ObsToGnav(files[0].c_str(),gnavfile)) files[2]=gnavfile;
	}
	AnsiString cmd="rtkplot -r \""+files[0]+"\" \""+files[1]+"\" \""+files[2]+"\"";
	if (!ExecCmd(cmd,1)) ShowMsg("error : rtkplot execution");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnInputPlot3Click(TObject *Sender)
{
	AnsiString files[4];
	char navfile[1024],gnavfile[1024];
	
	files[0]=FilePath(InputFile1->Text);
	files[1]=FilePath(InputFile2->Text);
	files[2]=FilePath(InputFile4->Text);
	files[3]=FilePath(InputFile3->Text);
	if (files[1]=="") {
		if (ObsToNav(files[0].c_str(),navfile)) files[1]=navfile;
	}
	if (files[2]=="") {
		if (ObsToGnav(files[0].c_str(),gnavfile)) files[2]=gnavfile;
	}
	AnsiString cmd="rtkplot -r \""+files[3]+"\" \""+files[1]+"\" \""+files[2]+"\"";
	if (!ExecCmd(cmd,1)) ShowMsg("error : rtkplot execution");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnPlotClick(TObject *Sender)
{
	AnsiString file=FilePath(OutputFile->Text);
	AnsiString cmd="rtkplot \""+file+"\"";
	if (!ExecCmd(cmd,1)) ShowMsg("error : rtkplot execution");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnViewClick(TObject *Sender)
{
	ViewFile(FilePath(OutputFile->Text));
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnToKMLClick(TObject *Sender)
{
	ConvDialog->Show();	
	ConvDialog->SetInput(FilePath(OutputFile->Text));	
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnOptionClick(TObject *Sender)
{
	if (OptDialog->ShowModal()!=mrOk) return;
	UpdateEnable();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnStopClick(TObject *Sender)
{
	showmsg("abort");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnExecClick(TObject *Sender)
{
	if (BtnExec->Caption=="Abort") {
		BtnExec->Enabled=false;
		return;
	}
	if (InputFile1->Text=="") {
		showmsg("error : no rover observation file");
		return;
	}
	if (InputFile3->Text==""&&PosMode>0) {
		showmsg("error : no base station observation file");
		return;
	}
	if (OutputFile->Text=="") {
		showmsg("error : no output file");
		return;
	}
	char *p;
	if (p=strrchr(OutputFile->Text.c_str(),'.')) {
		if (!strcmp(p,".obs")||!strcmp(p,".OBS")||!strcmp(p,".nav")||
			!strcmp(p,".NAV")||!strcmp(p,".gnav")||!strcmp(p,".GNAV")||
			!strcmp(p,".gz")||!strcmp(p,".Z")||
			!strcmp(p+3,"o")||!strcmp(p+3,"O")||!strcmp(p+3,"d")||
			!strcmp(p+3,"D")||!strcmp(p+3,"n")||!strcmp(p+3,"N")||
			!strcmp(p+3,"g")||!strcmp(p+3,"G")) {
			showmsg("error : output file extenstion error (%s)",p);
			return;
		}
	}
	setrov ("");
	setbase("");
	showmsg("");
	BtnExec->Caption="Abort";
	BtnExit->Enabled=false;
	BtnView->Enabled=false;
	BtnToKML->Enabled=false;
	BtnPlot->Enabled=false;
	BtnOption->Enabled=false;
	Panel1->Enabled=false;
	if (ExecProc()>=0) {
		AddHist(InputFile1);
		AddHist(InputFile2);
		AddHist(InputFile3);
		AddHist(InputFile4);
		AddHist(OutputFile);
	}
	if (strstr(Message->Caption.c_str(),"processing")) {
		setrov ("");
		setbase("");
		showmsg("done");
	}
	BtnExec->Caption="E&xecute";
	BtnExec->Enabled=true;
	BtnExit->Enabled=true;
	BtnView->Enabled=true;
	BtnToKML->Enabled=true;
	BtnPlot->Enabled=true;
	BtnOption->Enabled=true;
	Panel1->Enabled=true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnExitClick(TObject *Sender)
{
	Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::TimeStartClick(TObject *Sender)
{
	UpdateEnable();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::TimeIntFClick(TObject *Sender)
{
	UpdateEnable();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::TimeUnitFClick(TObject *Sender)
{
	UpdateEnable();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PosModeChange(TObject *Sender)
{
	UpdateEnable();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SolutionChange(TObject *Sender)
{
	UpdateEnable();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::RefPosClick(TObject *Sender)
{
	UpdateEnable();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnInputFile1Click(TObject *Sender)
{
	OpenDialog->Title="Observation Data File";
	OpenDialog->FileName="";
	OpenDialog->FilterIndex=1;
	if (!OpenDialog->Execute()) return;
	InputFile1->Text=OpenDialog->FileName;
	char file[1024],*p;
	strcpy(file,InputFile1->Text.c_str());
	if (!(p=strrchr(file,'.'))) p=file+strlen(file);
	strcpy(p,".pos");
	OutputFile->Text=file;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnInputFile2Click(TObject *Sender)
{
	OpenDialog->Title="Navigation Messages File";
	OpenDialog->FileName="";
	OpenDialog->FilterIndex=3;
	if (!OpenDialog->Execute()) return;
	InputFile2->Text=OpenDialog->FileName;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnInputFile4Click(TObject *Sender)
{
	OpenDialog->Title="Navigation Messages File";
	OpenDialog->FileName="";
	OpenDialog->FilterIndex=4;
	if (!OpenDialog->Execute()) return;
	InputFile4->Text=OpenDialog->FileName;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnInputFile3Click(TObject *Sender)
{
	OpenDialog->Title="Base Station Observation File";
	OpenDialog->FileName="";
	OpenDialog->FilterIndex=1;
	if (!OpenDialog->Execute()) return;
	InputFile3->Text=OpenDialog->FileName;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnOutputFileClick(TObject *Sender)
{
	SaveDialog->Title="Solution File";
	SaveDialog->FileName="";
	OpenDialog->FilterIndex=5;
	if (!SaveDialog->Execute()) return;
	OutputFile->Text=SaveDialog->FileName;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnInputView1Click(TObject *Sender)
{
	ViewFile(FilePath(InputFile1->Text));
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnInputView2Click(TObject *Sender)
{
	AnsiString file=FilePath(InputFile2->Text);
	char f[1024];
	if (file=="") {
		file=FilePath(InputFile1->Text);
		if (!ObsToNav(file.c_str(),f)) return;
		file=f;
	}
	ViewFile(file);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnInputView4Click(TObject *Sender)
{
	AnsiString file=FilePath(InputFile4->Text);
	char f[1024];
	if (file=="") {
		file=FilePath(InputFile1->Text);
		if (!ObsToGnav(file.c_str(),f)) return;
		file=f;
	}
	ViewFile(file);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnInputView3Click(TObject *Sender)
{
	ViewFile(FilePath(InputFile3->Text));
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnOutputView1Click(TObject *Sender)
{
	AnsiString file=FilePath(OutputFile->Text)+".stat";
	FILE *fp=fopen(file.c_str(),"r");
	if (fp) fclose(fp); else return;
	ViewFile(file);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnOutputView2Click(TObject *Sender)
{
	AnsiString file=FilePath(OutputFile->Text)+".trace";
	FILE *fp=fopen(file.c_str(),"r");
	if (fp) fclose(fp); else return;
	ViewFile(file);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SolFormatChange(TObject *Sender)
{
	UpdateEnable();
}
//---------------------------------------------------------------------------
int __fastcall TMainForm::ExecProc(void)
{
	gtime_t ts={0},te={0},t0={0},tts,tte;
	double tu=str2dbl(TimeUnit->Text)*3600.0;
	int stat=0,week,is;
	
	if (TimeStart->Checked) ts=GetTime1();
	if (TimeEnd  ->Checked) te=GetTime2();
	if (ts.time!=0&&te.time!=0&&TimeUnitF->Checked) {
		is=(int)floor(time2gpst(ts,&week)/tu);
		settspan(t0,t0);
		settspan(ts,te);
		for (int i=is;;i++) {
			tts=gpst2time(week,i*tu);
			tte=timeadd(tts,tu-0.01);
			if (timediff(tts,te)>0.0) break;
			if (timediff(tts,ts)<0.0) tts=ts;
			if (timediff(tte,te)>0.0) tte=te;
			if ((stat=ExecProc1(tts,tte))==1) break;
		}
		return stat;
	}
	return ExecProc1(ts,te);
}
//---------------------------------------------------------------------------
int __fastcall TMainForm::ExecProc1(gtime_t ts, gtime_t te)
{
	AnsiString s;
	char rov[256],*p,*q;
	int stat=0;
	
	if (strstr(InputFile1->Text.c_str(),"%r")||
		strstr(InputFile2->Text.c_str(),"%r")||
		strstr(InputFile3->Text.c_str(),"%r")||
		strstr(InputFile4->Text.c_str(),"%r")||
		strstr(OutputFile->Text.c_str(),"%r")) {
		
		if (RovList=="") {
			showmsg("error : no rover list");
			return 0;
		}
		for (p=RovList.c_str();*p;p=q+2) {
			if ((q=strstr(p,"\r\n"))) {
				strncpy(rov,p,q-p); rov[q-p]='\0';
			}
			else strcpy(rov,p);
			if (*rov&&rov[0]!='#') {
				setrov(rov);
				if ((stat=ExecProc2(ts,te,s=rov))==1) break;
			}
			if (!q) break;
		}
		return stat;
	}
	setrov("");
	return ExecProc2(ts,te,"");
}
//---------------------------------------------------------------------------
int __fastcall TMainForm::ExecProc2(gtime_t ts, gtime_t te, AnsiString rov)
{
	AnsiString s;
	char base[256],*p,*q;
	int stat=0;
	
	if (strstr(InputFile1->Text.c_str(),"%b")||
		strstr(InputFile2->Text.c_str(),"%b")||
		strstr(InputFile3->Text.c_str(),"%b")||
		strstr(InputFile4->Text.c_str(),"%b")||
		strstr(OutputFile->Text.c_str(),"%b")) {
		
		if (BaseList=="") {
			showmsg("error : no base station list");
			return 0;
		}
		for (p=BaseList.c_str();*p;p=q+2) {
			if ((q=strstr(p,"\r\n"))) {
				strncpy(base,p,q-p); base[q-p]='\0';
			}
			else strcpy(base,p);
			if (*base&&base[0]!='#') {
				setbase(base);
				if ((stat=ExecProcUnit(ts,te,rov,s=base))==1) break;
			}
			if (!q) break;
		}
		return stat;
	}
	setbase("");
	return ExecProcUnit(ts,te,rov,"");
}
//---------------------------------------------------------------------------
int __fastcall TMainForm::ExecProcUnit(gtime_t ts, gtime_t te,
	AnsiString rov, AnsiString base)
{
	AnsiString input1,input2,input3,input4,input5,output;
	int nfile=0,informat[4]={0};
	char *p,s1[4][1024]={{0}};
	char *infile[]={s1[0],s1[1],s1[2],s1[3]},outfile[256];
	double tint=0.0;
	prcopt_t prcopt=prcopt_default;
	solopt_t solopt=solopt_default;
	
	input1=RepFile(InputFile1->Text,ts,rov,base);
	input2=RepFile(InputFile2->Text,ts,rov,base);
	input3=RepFile(InputFile3->Text,ts,rov,base);
	input4=RepFile(InputFile4->Text,ts,rov,base);
	input5=RepFile(PrecEphFile,     ts,rov,base);
	output=RepFile(OutputFile->Text,ts,rov,base);
	
	strcpy(infile[nfile],input1.c_str());
	informat[nfile++]=STRFMT_RINEX;
	if (PosMode) {
		strcpy(infile[nfile],input3.c_str());
		informat[nfile++]=STRFMT_RINEX;
	}
	if (input2!="") {
		strcpy(infile[nfile],input2.c_str());
		informat[nfile++]=STRFMT_RINEX;
	}
	else {
		if (!ObsToNav(input1.c_str(),infile[nfile])) {
			showmsg("error : no gps navigation data : %s",input2.c_str());
			return 0;
		}
		informat[nfile++]=STRFMT_RINEX;
	}
	if (input4!="") {
		strcpy(infile[nfile],input4.c_str());
		informat[nfile++]=STRFMT_RINEX;
	}
	else {
		if (!ObsToGnav(input1.c_str(),infile[nfile])) {
			showmsg("error : no glonass navigation data : %s",input4.c_str());
			return 0;
		}
		informat[nfile++]=STRFMT_RINEX;
	}
	strcpy(outfile,output.c_str());
	
	if (TimeIntF->Checked) tint=str2dbl(TimeInt->Text);
	
	if (!GetOption(prcopt,solopt,input1,input3)) return 0;
	
	int dirs=prcopt.mode==0?0:Solution;
	
	if (!TimeStart->Checked||!TimeEnd->Checked||!TimeUnitF->Checked) {
		FILE *fp;
		if ((fp=fopen(outfile,"r"))) {
			fclose(fp);
			ConfDialog->Label2->Caption=outfile;
			if (ConfDialog->ShowModal()!=mrOk) return 0;
		}
	}
	if (DebugTrace>0) {
		char file[1024];
		sprintf(file,"%s.trace",outfile);
		traceopen(file);
		tracelevel(DebugTrace);
	}
	char file[1024];
	sprintf(file,"%s.stat",outfile);
	if (DebugStatus>0) {
		rtkopenstat(file,DebugStatus);
	}
	else {
		remove(file);
	}
	if (OutputGeoid>0&&GeoidDataFile!="") {
		opengeoid(OutputGeoid,GeoidDataFile.c_str());
	}
	postposopt(&prcopt,&solopt,input5.c_str(),SatPcvFile.c_str());
	
	showmsg("reading : %s...",input1.c_str());
	settime(ts);
	
	int stat=postpos(ts,te,tint,dirs,infile,nfile,outfile,informat);
	
	if (OutputGeoid>0&&GeoidDataFile!="") closegeoid();
	if (DebugTrace >0) traceclose();
	if (DebugStatus>0) rtkclosestat();
	
	if (stat==1) {
		setrov ("");
		setbase("");
		showmsg("aborted");
	}
	return stat;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ShowMsg(char *msg)
{
	Message->Caption=msg;
	Message->Font->Color=strstr(msg,"error")?clRed:clGray;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::UpdateEnable(void)
{
	InputFile3	  ->Enabled=PosMode>=1;
	BtnInputFile3 ->Enabled=PosMode>=1;
	BtnInputPlot3 ->Enabled=PosMode>=1;
	BtnInputView3 ->Enabled=PosMode>=1;
	BtnOutputView1->Enabled=DebugStatus>0;
	BtnOutputView2->Enabled=DebugTrace>0;
	LabelInputFile3->Enabled=PosMode>=1;
	TimeY1		  ->Enabled=TimeStart->Checked;
	TimeH1		  ->Enabled=TimeStart->Checked;
	TimeY1UD	  ->Enabled=TimeStart->Checked;
	TimeH1UD	  ->Enabled=TimeStart->Checked;
	BtnTime1	  ->Enabled=TimeStart->Checked;
	TimeY2		  ->Enabled=TimeEnd->Checked;
	TimeH2		  ->Enabled=TimeEnd->Checked;
	TimeY2UD	  ->Enabled=TimeEnd->Checked;
	TimeH2UD	  ->Enabled=TimeEnd->Checked;
	BtnTime2	  ->Enabled=TimeEnd->Checked;
	TimeInt		  ->Enabled=TimeIntF->Checked;
	LabelTimeInt  ->Enabled=TimeIntF->Checked;
	TimeUnitF	  ->Enabled=TimeStart->Checked&&TimeEnd->Checked;
	TimeUnit	  ->Enabled=TimeUnitF->Enabled&&TimeUnitF->Checked;
	LabelTimeUnit ->Enabled=TimeUnitF->Enabled&&TimeUnitF->Checked;
}
//---------------------------------------------------------------------------
int __fastcall TMainForm::GetOption(prcopt_t &prcopt, solopt_t &solopt,
	AnsiString input1, AnsiString input3)
{
	char buff[1024],*p;
	int sat;
	
	prcopt.mode=PosMode;
	prcopt.nf=Freq+1;
	prcopt.elmin=ElMask*D2R;
	prcopt.snrmin=SnrMask;
	prcopt.dynamics=DynamicModel;
	prcopt.navsys=NavSys;
	prcopt.sateph=SatEphem;
	prcopt.modear=AmbRes;
	prcopt.glomodear=GloAmbRes;
	prcopt.maxout=OutCntResetBias;
	prcopt.minlock=LockCntFixAmb;
	prcopt.elmaskar=ElMaskAR*D2R;
	prcopt.estiono=IonoEst;
	prcopt.esttrop=TropEst;
	prcopt.niter=NumIter;
	prcopt.codesmooth=CodeSmooth;
	prcopt.err[0]=MeasErr1;
	prcopt.err[1]=MeasErr2;
	prcopt.err[2]=MeasErr3;
	prcopt.err[3]=MeasErr4;
	prcopt.err[4]=MeasErr5;
	prcopt.prn[0]=PrNoise1;
	prcopt.prn[1]=PrNoise2;
	prcopt.prn[2]=PrNoise3;
	prcopt.prn[3]=PrNoise4;
	prcopt.prn[4]=PrNoise5;
	
	if (PosMode<PMODE_FIXED) {
		for (int i=0;i<3;i++) prcopt.ru[i]=0.0;
	}
	else if (RovPosType<=2) {
		for (int i=0;i<3;i++) prcopt.ru[i]=RovPos[i];
	}
	else if (!GetPos(input1.c_str(),prcopt.ru)) {
		showmsg("error : no rover pos in %s",StaPosFile.c_str());
		return 0;
	}
	for (int i=0;i<3;i++) prcopt.rb[i]=0.0;
	
	switch (RefPosType) {
		case 0: case 1: case 2: // fixed pos
			for (int i=0;i<3;i++) prcopt.rb[i]=RefPos[i];
			prcopt.refpos=0;
			break;
		case 3: // average of single pos
			prcopt.refpos=1;
			break;
		case 4: // get from pos file
	    	if (!GetPos(input3.c_str(),prcopt.rb)) {
				showmsg("error : no base position in %s",StaPosFile.c_str());
				return 0;
			}
			prcopt.refpos=0;
			break;
		case 5: // rinex approx pos
			prcopt.refpos=2;
			break;
	}
	for (int i=0;i<MAXSAT;i++) {
		prcopt.exsats[i]=0;
	}
	if (ExSats!="") { // excluded satellites
		strcpy(buff,ExSats.c_str());
		for (p=strtok(buff," ");p;p=strtok(NULL," ")) {
			if (!(sat=satid2no(p))) continue;
			prcopt.exsats[sat-1]=1;
		}
	}
	if (RovAntPcv&&RovAnt!="") {
		if (prcopt.pcvu) free(prcopt.pcvu);
		if (!(prcopt.pcvu=(pcv_t *)malloc(sizeof(pcv_t)))) return 0;
		if (readpcv(AntPcvFile.c_str(),RovAnt.c_str(),prcopt.pcvu)) {
			showmsg("error : no antenna pcv in %s",RovAnt.c_str());
		}
		prcopt.pcvu->ecc[0]=RovAntE;
		prcopt.pcvu->ecc[1]=RovAntN;
		prcopt.pcvu->ecc[2]=RovAntU;
	}
	if (RefAntPcv&&RefAnt!="") {
		if (prcopt.pcvr) free(prcopt.pcvr);
		if (!(prcopt.pcvr=(pcv_t *)malloc(sizeof(pcv_t)))) return 0;
		if (readpcv(AntPcvFile.c_str(),RefAnt.c_str(),prcopt.pcvr)) {
			showmsg("error : no antenna pcv : %s",RefAnt.c_str());
		}
		prcopt.pcvr->ecc[0]=RefAntE;
		prcopt.pcvr->ecc[1]=RefAntN;
		prcopt.pcvr->ecc[2]=RefAntU;
	}
	if (SbasCorr) {
		if (SbasCorr1) prcopt.sbascorr|=SBSOPT_LCORR;
		if (SbasCorr2) prcopt.sbascorr|=SBSOPT_ICORR;
		if (SbasCorr3) prcopt.sbascorr|=SBSOPT_FCORR;
		if (SbasCorr4) prcopt.sbascorr|=SBSOPT_RANGE;
	}
	prcopt.thresar=ValidThresAR;
	prcopt.thresslip=SlipThres;
	prcopt.maxtdiff=MaxAgeDiff;
	prcopt.maxinno=RejectThres;
	prcopt.intpref=IntpRefObs;
	if (BaseLineConst) {
		prcopt.baseline[0]=BaseLine[0];
		prcopt.baseline[1]=BaseLine[1];
	}
	else {
		prcopt.baseline[0]=0.0;
		prcopt.baseline[1]=0.0;
	}
	solopt.posf=SolFormat;
	solopt.times=TimeFormat==0?0:TimeFormat-1;
	solopt.timef=TimeFormat==0?0:1;
	solopt.timeu=TimeDecimal<=0?0:TimeDecimal;
	solopt.degf=LatLonFormat;
	solopt.outhead=OutputHead;
	solopt.outopt=OutputOpt;
	solopt.datum=OutputDatum;
	solopt.height=OutputHeight;
	solopt.geoid=OutputGeoid;
	strcpy(solopt.sep,FieldSep!=""?FieldSep.c_str():" ");
	sprintf(solopt.prog,"%s ver.%s",PRGNAME,VER_RTKLIB);
	return 1;
}
//---------------------------------------------------------------------------
int __fastcall TMainForm::ObsToNav(const char *obsfile, char *navfile)
{
	char *p;
	strcpy(navfile,obsfile);
	if (!(p=strrchr(navfile,'.'))) return 0;
	if		(strlen(p)==4&&*(p+3)=='o') *(p+3)='n';
	else if	(strlen(p)==4&&*(p+3)=='d') *(p+3)='n';
	else if (strlen(p)==4&&*(p+3)=='O') *(p+3)='N';
	else if (!strcmp(p,".obs")) strcpy(p,".nav");
	else if (!strcmp(p,".OBS")) strcpy(p,".NAV");
	else if (!strcmp(p,".gz")||!strcmp(p,".Z")) {
		if		(*(p-1)=='o') *(p-1)='n';
		else if	(*(p-1)=='d') *(p-1)='n';
		else if (*(p-1)=='O') *(p-1)='N';
		else return 0;
	}
	else return 0;
	return 1;
}
//---------------------------------------------------------------------------
int __fastcall TMainForm::ObsToGnav(const char *obsfile, char *gnavfile)
{
	char *p;
	strcpy(gnavfile,obsfile);
	if (!(p=strrchr(gnavfile,'.'))) return 0;
	if		(strlen(p)==4&&*(p+3)=='o') *(p+3)='g';
	else if	(strlen(p)==4&&*(p+3)=='d') *(p+3)='g';
	else if (strlen(p)==4&&*(p+3)=='O') *(p+3)='G';
	else if (!strcmp(p,".obs")) strcpy(p,".gnav");
	else if (!strcmp(p,".OBS")) strcpy(p,".GNAV");
	else if (!strcmp(p,".gz")||!strcmp(p,".Z")) {
		if		(*(p-1)=='o') *(p-1)='g';
		else if	(*(p-1)=='d') *(p-1)='g';
		else if (*(p-1)=='O') *(p-1)='G';
		else return 0;
	}
	else return 0;
	return 1;
}
//---------------------------------------------------------------------------
int __fastcall TMainForm::GetPos(char *file, double *rr)
{
	FILE *fp;
	char buff[256],code[256],*sta,*p,*q;
	double pos[3];
	int stat=0;
	if (!(fp=fopen(StaPosFile.c_str(),"r"))) return 0;
	if ((p=strrchr(file,'/'))||(p=strrchr(file,'\\'))) sta=p+1; else sta=file;
	while (fgets(buff,sizeof(buff),fp)) {
		if ((p=strchr(buff,'%'))) *p='\0';
		if (sscanf(buff,"%lf %lf %lf %s",pos,pos+1,pos+2,code)<4) continue;
		for (p=code,q=sta;*p&&*q;p++,q++) if (toupper(*p)!=toupper(*q)) break;
		if (*p) continue;
		pos[0]*=D2R; pos[1]*=D2R;
		pos2ecef(pos,rr);
		stat=1;
		break;
	}
	fclose(fp);
	return stat;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AddHist(TComboBox *combo)
{
	AnsiString hist=combo->Text;
	if (hist=="") return;
	TStrings *list=combo->Items;
	int i=list->IndexOf(hist);
	if (i>=0) list->Delete(i);
	list->Insert(0,hist);
	for (int i=list->Count-1;i>=MAXHIST;i--) list->Delete(i);
	combo->ItemIndex=0;
}
//---------------------------------------------------------------------------
TStringList * __fastcall TMainForm::ReadList(TIniFile *ini, AnsiString cat,
	AnsiString key)
{
	TStringList *list=new TStringList;
	for (int i=0;i<100;i++) {
		AnsiString s,item;
		item=ini->ReadString(cat,s.sprintf("%s_%03d",key.c_str(),i),"");
		if (item!="") list->Add(item); else break;
	}
	return list;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WriteList(TIniFile *ini, AnsiString cat,
	AnsiString key, TStrings *list)
{
	for (int i=0;i<list->Count;i++) {
		AnsiString s;
		ini->WriteString(cat,s.sprintf("%s_%03d",key.c_str(),i),list->Strings[i]);
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ViewFile(AnsiString file)
{
	char tmpfile[1024];
	if (file=="") return;
	int cstat=uncompress(file.c_str(),tmpfile);
	AnsiString f(!cstat?file.c_str():tmpfile);
	TTextViewer *viewer=new TTextViewer(Application);
	viewer->Caption=file;
	viewer->Show();
	viewer->Read(f);
	if (cstat==1) remove(tmpfile);
}
//---------------------------------------------------------------------------
int __fastcall TMainForm::ExecCmd(AnsiString cmd, int show)
{
	PROCESS_INFORMATION info;
	STARTUPINFO si={0};
	si.cb=sizeof(si);
	char *p=cmd.c_str();
	if (!CreateProcess(NULL,p,NULL,NULL,false,show?0:CREATE_NO_WINDOW,NULL,
					   NULL,&si,&info)) return 0;
	CloseHandle(info.hProcess);
	CloseHandle(info.hThread);
	return 1;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TMainForm::FilePath(AnsiString file)
{
	AnsiString rov,base;
	gtime_t ts={0};
	char buff[1024]="",*p,*q;
	
	if (TimeStart->Checked) ts=GetTime1();
	strcpy(buff,RovList.c_str());
	for (p=buff;(q=strstr(p,"\r\n"));p=q+2) {
		*q='\0';
		if (*p&&p[0]!='#') break;
	}
	rov=p;
	strcpy(buff,BaseList.c_str());
	for (p=buff;(q=strstr(p,"\r\n"));p=q+2) {
		*q='\0';
		if (*p&&p[0]!='#') break;
	}
	base=p;
	return RepFile(file,ts,rov,base);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TMainForm::RepFile(AnsiString file, gtime_t time,
	AnsiString rov, AnsiString base)
{
	AnsiString s;
	int week,dow,doy;
	double tow,ep[6],ep0[6]={2000,1,1,0,0,0};
	file=RepStr(file,"%r",rov.c_str());
	file=RepStr(file,"%b",base.c_str());
	if (time.time==0) return file;
	time2epoch(time,ep);
	ep0[0]=ep[0];
	dow=(int)floor(time2gpst(time,&week)/86400.0);
	doy=(int)floor(timediff(time,epoch2time(ep0))/86400.0)+1;
	file=RepStr(file,"%Y",s.sprintf("%04.0f",ep[0]));
	file=RepStr(file,"%y",s.sprintf("%02.0f",fmod(ep[0],100.0)));
	file=RepStr(file,"%m",s.sprintf("%02.0f",ep[1]));
	file=RepStr(file,"%d",s.sprintf("%02.0f",ep[2]));
	file=RepStr(file,"%n",s.sprintf("%03d",doy));
	file=RepStr(file,"%D",s.sprintf("%d",dow));
	file=RepStr(file,"%W",s.sprintf("%04d",week));
	file=RepStr(file,"%h",s.sprintf("%02d",(int)ep[3]));
	file=RepStr(file,"%H",s.sprintf("%c",'a'+(int)ep[3]));
	file=RepStr(file,"%f",s.sprintf("%d",(int)ep[3]/3+1));
	file=RepStr(file,"%t",s.sprintf("%02d",(int)ep[4]));
	return file;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TMainForm::RepStr(AnsiString str, AnsiString pat,
	AnsiString rep)
{
	char *p,*q,*r,buff[1024];
	for (p=str.c_str(),r=buff;*p;) {
		if (!(q=strstr(p,pat.c_str()))) break;
		strncpy(r,p,q-p); r+=q-p;
		strcpy(r,rep.c_str()); r+=strlen(r);
		p=q+strlen(pat.c_str());
	}
	strcpy(r,p);
	return (str=buff);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::TimeY1UDChangingEx(TObject *Sender,
	  bool &AllowChange, short NewValue, TUpDownDirection Direction)
{
	AnsiString s;
	double ep[]={2000,1,1,0,0,0};
	int p=TimeY1->SelStart,ud=Direction==updUp?1:-1;
	sscanf(TimeY1->Text.c_str(),"%lf/%lf/%lf",ep,ep+1,ep+2);
	if (4<p&&p<8) {
	    ep[1]+=ud;
	    if (ep[1]<=0) {ep[0]--; ep[1]+=12;}
	    else if (ep[1]>12) {ep[0]++; ep[1]-=12;}
	}
	else if (p>7||p==0) ep[2]+=ud; else ep[0]+=ud;
	time2epoch(epoch2time(ep),ep);
	TimeY1->Text=s.sprintf("%04.0f/%02.0f/%02.0f",ep[0],ep[1],ep[2]);
	TimeY1->SelStart=p>7||p==0?10:(p>4?7:4);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::TimeH1UDChangingEx(TObject *Sender,
	  bool &AllowChange, short NewValue, TUpDownDirection Direction)
{
	AnsiString s;
	int hms[3]={0},sec,p=TimeH1->SelStart,ud=Direction==updUp?1:-1;
	sscanf(TimeH1->Text.c_str(),"%d:%d:%d",hms,hms+1,hms+2);
	if (p>5||p==0) hms[2]+=ud; else if (p>2) hms[1]+=ud; else hms[0]+=ud;
	sec=hms[0]*3600+hms[1]*60+hms[2];
	if (sec<0) sec+=86400; else if (sec>=86400) sec-=86400;
	TimeH1->Text=s.sprintf("%02d:%02d:%02d",sec/3600,(sec%3600)/60,sec%60);
	TimeH1->SelStart=p>5||p==0?8:(p>2?5:2);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::TimeY2UDChangingEx(TObject *Sender,
	  bool &AllowChange, short NewValue, TUpDownDirection Direction)
{
	AnsiString s;
	double ep[]={2000,1,1,0,0,0};
	int p=TimeY2->SelStart,ud=Direction==updUp?1:-1;
	sscanf(TimeY2->Text.c_str(),"%lf/%lf/%lf",ep,ep+1,ep+2);
	if (4<p&&p<8) {
	    ep[1]+=ud;
	    if (ep[1]<=0) {ep[0]--; ep[1]+=12;}
	    else if (ep[1]>12) {ep[0]++; ep[1]-=12;}
	}
	else if (p>7||p==0) ep[2]+=ud; else ep[0]+=ud;
	time2epoch(epoch2time(ep),ep);
	TimeY2->Text=s.sprintf("%04.0f/%02.0f/%02.0f",ep[0],ep[1],ep[2]);
	TimeY2->SelStart=p>7||p==0?10:(p>4?7:4);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::TimeH2UDChangingEx(TObject *Sender,
	  bool &AllowChange, short NewValue, TUpDownDirection Direction)
{
	AnsiString s;
	int hms[3]={0},sec,p=TimeH2->SelStart,ud=Direction==updUp?1:-1;
	sscanf(TimeH2->Text.c_str(),"%d:%d:%d",hms,hms+1,hms+2);
	if (p>5||p==0) hms[2]+=ud; else if (p>2) hms[1]+=ud; else hms[0]+=ud;
	sec=hms[0]*3600+hms[1]*60+hms[2];
	if (sec<0) sec+=86400; else if (sec>=86400) sec-=86400;
	TimeH2->Text=s.sprintf("%02d:%02d:%02d",sec/3600,(sec%3600)/60,sec%60);
	TimeH2->SelStart=p>5||p==0?8:(p>2?5:2);
}
//---------------------------------------------------------------------------
gtime_t _fastcall TMainForm::GetTime1(void)
{
	double ep[]={2000,1,1,0,0,0};
	sscanf(TimeY1->Text.c_str(),"%lf/%lf/%lf",ep,ep+1,ep+2);
	sscanf(TimeH1->Text.c_str(),"%lf:%lf:%lf",ep+3,ep+4,ep+5);
	return epoch2time(ep);
}
//---------------------------------------------------------------------------
gtime_t _fastcall TMainForm::GetTime2(void)
{
	double ep[]={2000,1,1,0,0,0};
	sscanf(TimeY2->Text.c_str(),"%lf/%lf/%lf",ep,ep+1,ep+2);
	sscanf(TimeH2->Text.c_str(),"%lf:%lf:%lf",ep+3,ep+4,ep+5);
	return epoch2time(ep);
}
//---------------------------------------------------------------------------
void _fastcall TMainForm::SetTime1(gtime_t time)
{
	AnsiString s;
	double ep[6];
	time2epoch(time,ep);
	TimeY1->Text=s.sprintf("%04.0f/%02.0f/%02.0f",ep[0],ep[1],ep[2]);
	TimeH1->Text=s.sprintf("%02.0f:%02.0f:%02.0f",ep[3],ep[4],ep[5]);
	TimeY1->SelStart=10; TimeH1->SelStart=10;
}
//---------------------------------------------------------------------------
void _fastcall TMainForm::SetTime2(gtime_t time)
{
	AnsiString s;
	double ep[6];
	time2epoch(time,ep);
	TimeY2->Text=s.sprintf("%04.0f/%02.0f/%02.0f",ep[0],ep[1],ep[2]);
	TimeH2->Text=s.sprintf("%02.0f:%02.0f:%02.0f",ep[3],ep[4],ep[5]);
	TimeY2->SelStart=10; TimeH2->SelStart=10;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnTime1Click(TObject *Sender)
{
	TimeDialog->Time=GetTime1();
	TimeDialog->ShowModal();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnTime2Click(TObject *Sender)
{
	TimeDialog->Time=GetTime2();
	TimeDialog->ShowModal();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BtnAboutClick(TObject *Sender)
{
	AboutDialog->About=PRGNAME;
	AboutDialog->IconIndex=1;
	AboutDialog->ShowModal();
}
//---------------------------------------------------------------------------

